/*
 * udp-get.c: receive from UDP and send to stdout
 *
 * Copyright (c) 2000 Alessandro Rubini (rubini@gnu.org)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>

static int udp_bind(int sock, int port)
{
    struct sockaddr_in addr;

    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    addr.sin_addr.s_addr = INADDR_ANY;
    return bind(sock, (struct sockaddr *)&addr, sizeof(addr));
}

#define INBUF_SIZE 4096
char inbuf[INBUF_SIZE];

int main(int argc, char **argv)
{
    int sock, port, i, nb;
    struct sockaddr addr_me;

    if (argc != 2) {
	fprintf(stderr, "%s: Usage: \"%s <port>\"\n",
		argv[0], argv[0]);
	exit(1);
    }

    if (sscanf(argv[1], "%i%n", &port, &i) < 1
	|| i < strlen(argv[1]) || port <0 || port > 0xffff) {
	fprintf(stderr, "%s: \"%s\": not a valid port number\n",
		 argv[0], argv[1]);
    }
    sock =  socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0) {
	fprintf(stderr, "%s: socket(): %s\n", argv[0], strerror(errno));
	exit(1);
    }
    memset(&addr_me, 0, sizeof(addr_me));

    /* bind: try all ports in that range */
    if (udp_bind(sock, port)) {
	fprintf(stderr, "%s: cannot bind(): %s\n", argv[0], strerror(errno));
    }

    /* Done. Now receive */
    while (1) {
	nb = recvfrom(sock, inbuf, INBUF_SIZE, 0, NULL, NULL);
	if (nb < 0) {
	    fprintf(stderr, "%s: recvfrom(): %s\n",
		    argv[0], strerror(errno));
	    exit(1);
	}
	write(STDOUT_FILENO, inbuf, nb);
    }
    exit (0);
}
